from . import analysis
from . import brightness
from . import colordetection
from . import colorfulness
from . import contrast
from . import facedetection
from . import linesdetection
from . import quadtreedecomposition
from . import saturation
from . import selfsimilarity
from . import spacebaseddecomposition
from . import symmetry
from . import utils
from . import visualcomplexity

# This is used to print out the package version
__version__ = '0.0.8.4' #Version Control

print("Thank you for using pyaesthetics. If you use it in your work, please cite:")
print("Gabrieli, G., Bornstein, M. H., Setoh, P., & Esposito, G. (2023). Machine learning estimation of users’ implicit and explicit aesthetic judgments of web-pages. Behaviour & Information Technology, 42(4), 392-402.")
